PRO FOCIS, ACQFILE=acqfile, VIGNET=vignet, CBW=cbw, BLOCKNS=blockns, CHANNELS=channels, FILTERPARM=filterparm, $
			DISPLAY_SIZE=display_size, OVERVIEW_PLOTS=overview_plots, PS=ps, DYN_SPECTRA=dynamic_spectra, INTEGTIME=integtime, $
			FOCISOUT=focisout, PLOT=plot, OUTFILE=outfile, BWOUT=bwout, NOPSYS=nopsys, NOGEOM=nogeom, NOREFORMAT=noreformat, $
			KURTOSIS_FLAG=kurtosis_flag
;
; FASR Offline Correlator Implemented in Software
;
; Developed for FASR SUBSYSTEM TESTBED (FST) project
;
; Keywords:
;	/VIGNET 		uses saved vout data in fst_vignet.sav as input.  (NOT YET OPERATIONAL)
; 	ACQFILE			= acqfile specifies input file created by Acqiris system.
;					If neither ACQFILE or /VIGNET is specified, a choice of ACQ files in the default directory will be offered.
;
;	CBW 			= correlator bandwidth (MHz).  Default = 1.
; 	BLOCKNS			= duration of a block in ns. (A block forms the input for a single FFT).  Overridden by CBW.
;	CHANNELS		= array of channel numbers (1,2,3,4) to be processed.  	Default=[1,2,3]
;	FILTERPARM 		= a structure with filter parameters.  Default = no filtering.
;   INTEGTIME       = integration time for post-correlator accumulators [ms].  DEFAULT=1 segment.
;   BWOUT           = approximate bandwidth of post-correlation accumulators [MHz].  Default is no post-correlation frequency averaging.
;
;   FOCISOUT        = names an output structure to receive the results of the analysis.
;   OUTFILE         = name of binary file in which output will be saved.  Default = 'focisout.dat'.
;   /NOREFORMAT     suppresses reformating of interim, focisoutemp.dat file to outfile version.  Default is to reformat.
;
;   /PLOT           generates output plots immediately.  Default is no display.
;	DISPLAY_SIZE 	= 2-element vector defining x,y fraction of screen to use for dynamic spectrum.
;	/OVERVIEW_PLOTS generates overview plots of histograms, frequency-integrated light curves and time-integrated spectra
;	/DYN_SPECTRA    generates dynamic spectra plots.  Default=no dynamic spectra
;	/PS 			directs overview plots to a postscript file, idl.ps.
;						Default=[0.625,0.625]
;
;  /NOPSYS          suppresses normalization to PSYS and PSYSATTEN values.  Default is to do the normalization.
;  /NOGEOM          suppresses processing of geometry information, as with lab data.
;
; Optional outputs: 	Plotman displays of dynamic spectra of total power, correlated amplitude and phase.
;						6 overview plots for total power and correlated power:
;							In the form of histograms, spectrum-averaged light curves and time-averaged spectra.
;
; Restrictions:     No provision for scans that span a change of day.
;
; Modules used:
;   fst_accumulate
;	fst_filter
;	fst_header
;	fst_template
;	fst_byte2int
;   fst_sim_setup
;	fst_process_analysis_block
;	fst_overview_plots
;	fst_spectrogram_display
;   focis_display
;   fst_focisout_write
;   fst_focisout_reformat
;
; 15-Mar-2005 gh	Initial shell.  (ghurford@ssl.berkeley.edu)
; 17-Mar-2005 gh	Add display call to spectrogram object
; 18-Mar-2005 gh	Continue development
; 13-May-2005 gh	Adapt to use new ACQFILE input
; 16-May-2005 gh	Adapt to use new spectrogram interface protocol.
;					Change default block duration to correspond to 1 segment
;					Print processing parameter summary
;					Break out block processing and spectrogram display to separate modules.
; 21-Jun-2005 gh	Correct bug that compromised time axis
; 22-Jun-2005 gh	Add CHANNELS AND DISPLAY_SIZE keywords.
;					Add provision for multichannel display.
; 23-Jun-2005 gh	Add provision for overview plots.
;					Add PS keyword.
;					Update simulation input handling.
; 24-Jun-2005 gh	Process power spectrum instead of amplitude spectrum
; 27-Jun-2005 gh/gn	Correct bug which neglected Labview/IDL byte conversion
;  7-Jul-2005 gh	Update ACQ package module names.
; 11-Aug-2005 gh	Reorganize block & channel loops to support correlation provisions
; 16-Aug-2005 gh	Correct .ps color plot bug.
;					Include basic cross-correlation analysis and display
;					Include timing displays
; 17-Aug-05 gh		Add NODS keyword.
;					Distinguish single and multichannel colors in summary display
; 26-Aug-05	gh		Add CBW keyword.
;  5-Sep-05 gh		Add FILTERPARM keyword and optional calls to fst_filter
;					Changes to support improved overview plot labelling
;  7-Sep-05 gh		Clarify defaulting between cbw and blockns
;  1-Nov-05 gh		Change phase display range to [-180, +180]
; 21-Nov-05 gh		Create a focusout structure that can be saved or used for display input.
; 23-Feb-06 gh      Add provision for post-correlation integration.
; 24-Feb-06 gh      Correct transparent bug that failed to set default for CHANNELS.
;                   Break out all plot display code into FOCIS_DISPLAY to support post analysis.
;                   Add /PLOT keyword to generate immediate display.
; 25-Feb-06 gh      Add SAVEFILE keyword.  Results are SAVEd to savefile.  Default='focisout.sav'
;                   Add BWOUT keyword to support post-correlation frequency-averaging.
;                   Debug provision for 1-channel data
;                   Replace NODS keyword with DYNAMIC_SPECTRA keyword
; 26-Feb-06 gh      Express results in terms of RF rather than IF frequency.
;                   Normalize spectra to PSYS and PSYSATTEN values.
;                   Remove call to GET_LUN before fst_header which apparently has an undocumented feature that gets its own unit number.
;                   Print scan description to output log.
;  9-Mar-06 gh/dg   Use LONG instead of FIX when defining nfreqout to alow for larger spectra.
; 11-Mar-06 gh      Use overwrite switch when reforming large arrays.
;  5-May-06 gh      Test for existance of 'band' tag in datin
;  9-May-06 gh      Test for existance of psysatten structure in datin and bypass normalization if missing
; 10-May-06 dg/gh   Express results in terms of absolute, not relative time.
;                   Add provision for a hardcoded geometrical delay calculation
; 11-May-06 gh      FIX rather than ROUND calculation of nblockperintegtime to avoid truncated blocks
;                   Change output structure timetag from relative milliseconds to absolute seconds (in day).
; 15-May-06 gh      Write output data as a binary rather than a .sav file.
; 16-May-06 gh      Reorganize to write output every record rather than every file.
;                   Plotting options temporarily suppressed.
;                   Replace SAVEFILE keyword with OUTFILE keyword
; 17-May-06 gh      Debug 16-may-06 changes.
; 19-May-06 gh      Default calls fst_focisout_reformat to convert the incrementally-generated output file to the original version.
;                   Add /NOREFORMAT switch to suppress this feature
; 25-May-06 gh      Move interactive selection of input file(s) to FOCIS instead of fst_header.
;                   Display input filename.
;                   Suppress normalization if /NOPSYS is set or if psys tags are missing.
; 26-May-06 gh      Correct bug which suppressed direct return of focisout via fst_focisout_reformat.
;                   Incomplete implementation of KURTOSIS_FLAG keyword
;
;
DEFAULT, display_size,  [0.625, 0.625]
DEFAULT, ps,            0
DEFAULT, outfile,      'focisout.dat'
DEFAULT, nopsys,        0                           ; default is to do the normalization
DEFAULT, nogeom,        0                           ; default is to do geometric corrections
DEFAULT, bandswitch,    0                           ; default for early data where it was not specified
DEFAULT, noreformat,    0
;
iap			= [[0,1], [1,2], [2,0]]					; indices for antenna pairs.
;
; Get header and data structure for either simulated or ACQ-generated data.
IF KEYWORD_SET(vignet) THEN BEGIN
	MESSAGE, 'Sorry, the /VIGNET option is still under construction.'
	fst_sim_setup, acqheader, datin					; Simulated data
ENDIF ELSE BEGIN									; ACQ-generated data
    IF N_ELEMENTS(acqfile) EQ 0 THEN acqfile = DIALOG_PICKFILE(FILTER='ACQ*.dat')       ; Choose acqfile(s) interactively
    acqheader   = fst_header(acqfile, UNIT=acqlun, /notfree)
	datin		= fst_template(acqheader)           ; datin is an 'empty' structure with the correct tags, but no data
ENDELSE
tsys1 = SYSTIME(/SECONDS)
;
; Extract and convert header data.
nchan 		= acqheader.nchan						; number of data channels
nrec		= acqheader.nrec						; records in file
nsegperec	= acqheader.nseg						; segments per record
nsamperseg	= acqheader.nsamperseg					; samples per segment
samptimens	= acqheader.samptime * 1.E9				; sampling interval (ns)
DEFAULT, channels, INDGEN(acqheader.nchan)+1		; channel numbers to be processed (=1,2,3 and/or 4)
nch			= N_ELEMENTS(channels) < nchan			; number of channels to be processed
ich			= channels -1							; vector of indices for channels to be processed
nap			= nch*(nch-1)/2							; number of antenna pairs
;
; Determine processing parameters
IF N_ELEMENTS(cbw) EQ 0 AND N_ELEMENTS(blockns) EQ 1 THEN cbw = 1000. / blockns
IF N_ELEMENTS(cbw) EQ 0 THEN cbw = 0.9765625				    ; ~1 MHz ==> block = 1024 ns
IF cbw EQ 0 THEN BEGIN
	blockns = nsamperseg * samptimens 				            ; Minimum possible CBW ==> block = 1 segment
	cbw = 1000. / blockns
ENDIF ELSE BEGIN
	blockns = 1000. / cbw
ENDELSE
;
;
DEFAULT, integtime, samptimens * nsamperseg * 1.E-6                         ; Default integration time [ms] corresponds to 1 segment
DEFAULT, bwout,     cbw
nsamperblock 	    = LONG(blockns / samptimens)				            ; number of samples per block
nblockperseg	    = nsamperseg / nsamperblock			 		            ; number of blocks per segment
nblockperec		    = nblockperseg * nsegperec					            ; number of blocks per record
nblockperfile       = nrec * nblockperec
nblockperintegtime  = FIX(integtime * 1.E6 / blockns)                     ; number of blocks per post-correlation integration time
IF nblockperseg       LT 1 THEN MESSAGE, 'FOCIS IS STOPPING. BLOCK DURATION EXCEEDS SEGMENT DURATION'
IF nblockperintegtime LT 1 THEN MESSAGE, 'FOCIS IS STOPPING. BLOCK DURATION EXCEEDS INTEGRATION TIME'
nfreq		        = nsamperblock/2 -1   									; number of frequency points
df 			        = 1. / blockns											; frequency step & lowest frequency (GHz)
ntimeoutinrec        = FIX(nblockperec / nblockperintegtime)               ; number of time points in record after accumulation
ntimeoutinfile      = ntimeoutinrec * nrec
abstime             = DBLARR(ntimeoutinrec)                                      ; will hold absolute time of data points [s]
ncorchanperoutchan  = ROUND(bwout/cbw) > 1                                  ; number of correlator output frequencies per accumulator output channel
;nfreqout            = LONG(nfreq / ncorchanperoutchan)                     ; number of frequency points after accumulation
nfreqout            = LONG(nfreq / ncorchanperoutchan)                      ; number of frequency points after accumulation
dfout               = df * ncorchanperoutchan * 1.E3		        	    ; output frequency increment in MHz;
; COMMENTS
; Note that the driving parameter is the user-specified block duration(ns).
;	This determines the frequency resolution and also determines the amount of
;		unanalyzed data (if any) that is left over at the end of a segment.
;
; Get geometry information, provided it is wanted.
IF nogeom EQ 0 THEN IF tag_exist(acqheader, 'geometry', /QUIET) EQ 0 THEN $
    RESTORE, dialog_pickfile(filter='*.sav', TITLE='Select GEOMETRY .sav file') $        ; For use if geometry data is missing
    ELSE geometry = acqheader.geometry
;
; Display processing parameters
PRINT
PRINT, acqheader.scanid, 			FORMAT="('ScanID:     ', A)"
PRINT, acqfile,                     FORMAT="('ACQfile:    ', A)"
PRINT, acqheader.scandesc,          FORMAT="('ScanDesc:   ', A)"
PRINT, samptimens, 					FORMAT="('Input:      ', F8.1, ' ns per sample')"
PRINT, nsamperseg, 					FORMAT="(12X,            I8,   ' samples/segment')"
PRINT, nsegperec, 					FORMAT="(12X,            I8,   ' segments/record')"
PRINT, nrec, 						FORMAT="(12X,            I8,   ' records/file')"
PRINT, nchan,                       FORMAT="(12X,            I8,   ' channel data')"
PRINT, nsamperblock,                FORMAT="('Correlator: ', I8,   ' samples per block')"
PRINT, nblockperseg, 				FORMAT="(12X,            I8,   ' blocks/segment')"
PRINT, nblockperintegtime,          FORMAT="('Accumulator:', I6,   ':1 time compression')"
PRINT, ncorchanperoutchan,          FORMAT="(12X,            I6,   ':1 frequency compression')"
PRINT
PRINT, integtime,  dfout, 	        FORMAT="(F8.3, ' millisecond time bins with', F7.3,' MHz frequency resolution')"
;
; Create correlator output arrays
pwrspect0 	                = REFORM( FLTARR   (nblockperec, nfreq, nch), nblockperec, nfreq, nch, /OVERWRITE)				; power spectrum
IF nap GT 0 THEN corspect0	= REFORM(COMPLEXARR(nblockperec, nfreq, nap), nblockperec, nfreq, nap, /OVERWRITE)			    ; correlated visibility spectrum
;
; Begin outer loop over input records
tsys2 = SYSTIME(/SECONDS)
FOR nr = 0, nrec-1 DO BEGIN
	READU, acqlun, datin
;
; Calculate frequencies for this record.
    IF nr EQ 0 THEN frange = [99999., 0.]                                      ; Will hold min, max frequencies
    lomhz           = datin.lofreq
    bandswitch      = 0                                                                 ; default for 2005 data
    IF TAG_EXIST(datin,'band') THEN bandswitch = datin.band                             ; =1,0 for upper and lower sideband of LO
    sidebandswitch  = datin.sideband                                                    ; =1,0 for 5-9 and 1-5 GHz    (comments corrected per 20060313 gn email)
    IF bandswitch EQ sidebandswitch THEN ifsense = 1 ELSE ifsense = -1                  ; =1,-1 if RF and IF frequencies have same,different sense
    basefreq        = (10000*(1-bandswitch)*(1-2*sidebandswitch) - lomhz*(1-2*sidebandswitch) -1000) * ifsense  ; =RF frequency for IFfreq=0
    f0          = basefreq + ifsense * dfout / 2.                                            ; center of lowest post-accumulator frequency channel
    freqmhz		= f0 + FINDGEN(nfreqout)*dfout*ifsense
    frange[0] = MIN([frange[0], ABS(basefreq), ABS(basefreq+ifsense*500)])
    frange[1] = MAX([frange[1], ABS(basefreq), ABS(basefreq+ifsense*500)])

;
; Determine the power level for each channel.
   IF nr EQ 0 AND tag_exist(datin, psysatten, /QUIET) EQ 0 THEN nopsys = 1        ; Bypass normalization for 2005 data
   IF nopsys EQ 0 THEN BEGIN
            psys1       = datin.psys.data                           ; dbm       nseg x nchan array
            psysatten1  = datin.psysatten.data                      ; db        nseg x nchan array
            psysatten   = REPLICATE(1, nsegperec)#psysatten1        ; db        nseg x nchan array ???????? COPIED DG MODS
            psys        = 10.^( (psys1+psysatten)*0.1)              ; milliwatts per 500 MHz before attenuator. nseg x nchan  array
    ENDIF ELSE psys = FLTARR(nsegperec, nchan) -1                   ; If nopsys, then set psys values to -1 to act as a flag
;       PRINT, datin.psys.timestamp, datin.psysatten.data, datin.psys.data, FORMAT="(2F20.6, 20E12.4)"
;
; Set up acquisition-related times and delays as needed
    juldaytime = julday(1,1,1904,0,0,0) + datin.acqstart/86400d                                         ; Julian day of start of acquisition
    acqtimems = ((juldaytime - 0.5) MOD 1d) * 86400000d                                                 ; time in day of acquistion start in millseconds
    tseg = (ISHFT(ULONG(datin.segdesc.timestamphi), 32) + ULONG(datin.segdesc.timestamplo))*1.D-12 + acqtimems*1D-3    ; start time of segment (seconds)

    blocktimes = FINDGEN(ntimeoutinrec) * integtime*1.D-3                                                      ; block times relative to start of segment (ms)
    IF nogeom EQ 0 THEN delay, acqtimems, 1., tau, taudot, geometry, uv                                 ; Calculate tau, taudot, uv for acquisition start
;
; Begin intermediate loop over segments in the record
	FOR ns = 0, nsegperec-1 DO BEGIN
        abstime = tseg[ns] + blocktimes         ; absolute time of datapoints ?????? DG
		ns1 = ns * nsamperseg
		ns2 = ns1 + nsamperseg - 1
		segbytes = datin.data[ns1:ns2, ich]							; data bytes in this segment
;
; Bit truncation goes here
;
; Optional delay shifting.   DG code assumes ant 7 is ref antenna and that ch 0,1 are ant 5 and 6.  959 is a delay center value for both 5 and 6.
; Note also that shifts are circular - this should be changed
        IF nogeom EQ 0 THEN BEGIN
            segbytes[*, ich[0]] = SHIFT(segbytes[*, ich[0]], -tau[3,5])
            segbytes[*, ich[1]] = SHIFT(segbytes[*, ich[1]], -tau[4,5] - 959)
        ENDIF
;
; Apply optional filtering to one segment
		IF N_ELEMENTS(filterparm) NE 0 THEN  segbytes = fst_filter(segbytes, acqheader, filterparm)
;
; Begin	inner loop over analysis blocks
		FOR nbrel = 0, nblockperseg-1 DO BEGIN
			nb					= nbrel + ns*nblockperseg	; pointer for storing analysis output
			n1 					= nbrel * nsamperblock
			n2 					= n1 + nsamperblock-1
			block 				= fst_byte2int(segbytes[n1:n2,*])	; --> nsamperblock x nch  array
            IF nch EQ 1 THEN block = REFORM(block, nsamperblock, nch, /OVERWRITE)
			fst_process_analysis_block, block, power_out, correl_out, psys[ns,*]
			pwrspect0[nb,*,*] 	=  power_out[1:*,*]			                ; Exclude zero frequencies --> pwrspect[block, freq, channel]
			IF nap GT 0 THEN corspect0[nb,*,*] 	= correl_out[1:*,*]			;   and correlator spectrum [block, freq, antpair]
		ENDFOR
	ENDFOR
;
; Post-correlation accumulation for one record.
    pwrspect = fst_accumulate (pwrspect0, nblockperintegtime, ncorchanperoutchan, KSPECT=kspect )
    IF nap GT 0 THEN corspect = fst_accumulate (corspect0, nblockperintegtime, ncorchanperoutchan )
    IF nap EQ 0 THEN corspect = 0           ; Dummy value for case of single channel data.
;
;  Write output for one record to output file.
IF KEYWORD_SET(kurtosis_flag) THEN pwrspect=kspect
    focisout = {acqheader:	acqheader,	$
    			timesec: 	abstime, 	$
    			freqmhz: 	freqmhz, 	$
    			pwrspect: 	pwrspect, 	$
    			corspect: 	corspect 	}
    newfileswitch = 0
    IF nr EQ 0 THEN newfileswitch = 1
    fst_focisout_write, luntemp, focisout, HEADER=newfileswitch, NEWFILE=newfileswitch, FILENAME='focisoutemp.dat'     ; Write interim file
ENDFOR
outfilestat = FSTAT(luntemp)       ; Get an estimate of output file size
CLOSE, luntemp
FREE_LUN, luntemp
FREE_LUN, acqlun
PRINT, frange, FORMAT= "('Overall frequency range:',  I5, ' to', I5, ' MHz')"
tsys3 = SYSTIME(/SECONDS)


;
; Generate displays.
; timems  = timetab     ; OBSOLETE
PRINT
PRINT, nch, ntimeoutinfile, nfreq, FORMAT="(I2, ' channel dynamic spectra each have ', I6, ' time and ', I7, ' frequency points.')"
;PRINT
;
; Create a structure containing analysis results.
;
; Save and optionally display results.
;SAVE, focisout, FILE=savefile

;IF KEYWORD_SET(plot) NE 0 OR KEYWORD_SET(dynamic_spectra) NE 0 OR KEYWORD_SET(overview_plots) NE 0 THEN $
;    FOCIS_DISPLAY, focisout,  DISPLAY_SIZE=display_size, OVERVIEW_PLOTS=overview_plots, PS=ps, DYN_SPECTRA=dyn_spectra, CHANNELS=channels
;
; Elapsed time summary
PRINT
PRINT, tsys3-tsys1, FORMAT="('Processing time: ', F6.1, ' seconds')"
PRINT, outfilestat.size/1.E6,  FORMAT='("    Interim results written to focistemp.dat ", F12.6, " MBytes")'
;
; Reformat interim output from a record-oriented to a file-oriented format.
IF noreformat EQ 0 THEN fst_focisout_reformat, OUTFILE=outfile, FOCISOUT=focisout
RETURN
END

;
; Bugs, improvements and next features
;
; Bugs:
;       Resolve normalization inconsistency between input and spectrogram display.
;
; Improvements:
; *****	Need default/option for compressed spectra to show maximum, not average value within a displayed pixel.
;		Display option should show extreme instead of average value of compressed pixels.
;     	Normalize output to dbm
;
; Next:
;       Provision to reformat multiband files
;       Provision to process multifile scans
;   	Add closure phase plots
;		Use different symbols for oplots
;		Fix up amplitude summary plot labelling
;		Incorporate poly-phase prefilters.
;       Improve delay handling - generalize and center correlation time.
; 		Adapt VIGNET to output files in a more ACQ-like format.
;
;		Add /binary switch
;		Need to force timeblocks to be even in size