FUNCTION FST_ACCUMULATE, spect, ntav, nfav, KSPECT=kspect
;
; Averages input spectrum over time and/or frequency points.
;
; spect is an nblock x nfreq x n  real or complex array.
; Returns an   ntime x nfrq  x n  array of the same type.
;
; 23-Feb-06     Initial version (ghurford@ssl.berkeley.edu)
; 24-Feb-06 gh  Add provision for averaging over frequency as well.
; 26-May-06 gh  Add KSPECT keyword to receive optionally returned kurtosis spectrum, assuming spect is REAL.
;
; nfav is optional
;
sizedat = SIZE(spect)
nc      = sizedat[3]
nfreq   = sizedat[2]
DEFAULT, kurtosis_flag, 0
IF ARG_PRESENT(kspect) NE 0 AND sizedat[4] EQ 4 AND ntav GT 4 THEN kurtosis_flag = 1       ; Only calculate kurtosis for total power averaging > 4 pts
;
; Average over time, if necessary
nblock = sizedat[1]
ntpt = FIX(nblock / ntav)            ; number of output time points
IF ntav EQ 1 THEN tavspect = avspect $
ELSE BEGIN
    IF sizedat[4] EQ 4 THEN tavspect = REFORM (FLTARR    (ntpt, nfreq, nc), ntpt, nfreq, nc)   $
                              ELSE tavspect = REFORM (COMPLEXARR(ntpt, nfreq, nc), ntpt, nfreq, nc)
    IF kurtosis_flag THEN kspect = tavspect               ; array to hold kurtosis results
    FOR i = 0, ntpt-1 DO BEGIN
        j1              = i * ntav
        j2              = j1 + ntav - 1
        tavspect[i,*,*] = TOTAL(spect[j1:j2, *,*],1)/ntav
        IF kurtosis_flag THEN FOR n = 0, nc-1 DO FOR m = 0, nfreq-1 DO kspect[i,m,n] = KURTOSIS(spect[j1:j2,m,n])
    ENDFOR
ENDELSE
;
; Average over frequency, if necessary
IF nfav EQ 1 OR N_ELEMENTS(nfav) EQ 0 THEN RETURN, tavspect              ; Return immediately if no frequency averaging is necessary.
IF kurtosis_flag THEN tavkspect=kspect
nfpt = FIX(nfreq / nfav)             ; number of output frequency points
IF sizedat[4] EQ 4 THEN avspect = REFORM (FLTARR    (ntpt, nfpt, nc), ntpt, nfpt, nc)   $
                   ELSE avspect = REFORM (COMPLEXARR(ntpt, nfpt, nc), ntpt, nfpt, nc)
IF kurtosis_flag THEN kspect = avspect
FOR i = 0, nfpt-1 DO BEGIN
    j1              = i * nfav
    j2              = j1 + nfav - 1
    avspect[*,i,*]  = TOTAL(tavspect[*,j1:j2,*],2)/nfav
    IF kurtosis_flag THEN FOR n = 0, nc-1 DO FOR m = 0, nfpt-1 DO kspect[m,i,n] = KURTOSIS(spect[m,j1:j2,n])
ENDFOR
RETURN, avspect
END

