;+
; NAME:
;     fst _dswrite
; PURPOSE:
;     To write an IDL variable to a LabVIEW data socket
; CATEGORY:
;     OVRO FST DATA ANALYSIS
; CALLING SEQUENCE:
;     error=fst_dswrite(data,attributes=attributes,target=target,timeout=timeout,path=path)
; INPUTS:
;     data: an IDL variable
;
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
;     attributes: an IDL structure whose tags to be added as variant attributes
;     target- datasocket URL
;             if not provided it is set to the default 'dstp://localhost/lv2idl'
;     bytestr- an array of bytes to store the number of bytes to be read.
;             if not provided it is set to a default [0b,0b,0b,0b]
;     timeout- ms to wait for the operation to be completed (default 1000ms)
;     path- the complete path of the DS.DLL external library, if not local
;
; ROUTINES CALLED:
;     fst_flatten2bytestr,fst_call_dswrite
; OUTPUTS:
;  return value: error code
;                0 - no error,
;                56-  The network operation exceeded
;                     the user-specified or system time limit.
;                63-  The network connection was refused by the server.
; COMMENTS:
;
; SIDE EFFECTS:
; RESTRICTIONS:
;     some IDL data types might not be yet supported
; MODIFICATION HISTORY:
;     Written March 31-2005 by Gelu Nita (gnita@njit.edu)
function fst_dswrite,data,attributes=attributes,target=target,timeout=timeout,path=path
	bytestr=fst_flatten2bytestr(data,attributes=attributes)
	return,fst_call_dswrite(target=target,bytestr=bytestr,timeout=timeout,path=path)
end