FUNCTION FST_FILTER, segbytes, acqheader,  filterparm
;
; Returns a filtered version of a segment of input bytes
;
; segbytes = nsamperseg x nchan array of input bytes
;
;  5-Sep-05 gh	Initial version (ghurford@ssl.berkeley.edu)
;				Simply sets offending frequency channels to zero.
;
; Set up filtering parameters
IF filterparm.fmode EQ 0 THEN RETURN, segbytes
nsamperseg 		= acqheader.nsamperseg					; number of samples per segment
samptimens		= acqheader.samptime * 1.E9				; sample time in nanoseconds
fbw				= filterparm.fbw						; filterbandwidth (MHz)
fblockns		= 1000. / fbw							; filterblocksize (ns)
nsamperfblock 	= LONG(fblockns/samptimens)				; number of samples per filterblock
nfblockperseg 	= nsamperseg / nsamperfblock			; number of filterblocks per segment
nfreq 			= nsamperfblock/2
df				= 1./fblockns * 1.E3					; frequency step and lowest frequency(MHz)
;
; Assumed structure of filterparameters
; filterparm = {fmode:		1,					$		; 0 = no filtering, -1=auto, 1 prespecified frequencies
;				fbw: 		0,					$
;				nbadfreq: 	0, 					$
;				badfreq:	FLTARR(nbadfreq), 	$		; MHz
;				badfreqbw: 	FLTARR(nbadfreq) }			; MHz
;
; Identify channels to be rejected
IF filterparm.fmode EQ 1 AND filterparm.nbadfreq EQ 0 THEN RETURN, segbytes
IF         (N_ELEMENTS(filterparm.badfreq)   NE filterparm.nbadfreq) $
		OR (N_ELEMENTS(filterparm.badfreqbw) NE filterparm.nbadfreq) THEN MESSAGE, 'Bad filterparameters'
rejectchan	= INTARR(nsamperfblock)											; Default, 0 ==> do not reject
FOR n = 0, filterparm.nbadfreq-1 DO BEGIN
	clo = FIX((filterparm.badfreq[n] - filterparm.badfreqbw[n]/2) / df)     > 1			; lowest channel to reject
	IF clo GT nfreq THEN CONTINUE														; Skip to next bad frequency
	chi = CEIL((filterparm.badfreq[n] + filterparm.badfreqbw[n]/2) / df) < nfreq		; highest channel to reject
	rejectchan[clo:chi] = 1
	rejectchan[(nsamperfblock-chi):(nsamperfblock-clo)] = 1								; reject the image channels too
ENDFOR
rejectchantab 	= WHERE(rejectchan NE 0, nrejectchan)		; table of channels to reject
IF nrejectchan EQ 0 THEN RETURN, segbytes					; No need to continue if no channels to reject
;
; Begin loop over filterblocks
FOR n = 0, nfblockperseg-1 DO BEGIN
	n1 = n*nsamperfblock
	n2 = n1 + nsamperfblock - 1
	fblock = fst_byte2int(segbytes[n1:n2, *])								; -> nsamperfblock x nchan array
	fblockout = fst_process_filterblock(fblock, rejectchantab)				; Replaces fblock with filtered version
	segbytes[n1:n2, *] = fst_float2byte(fblockout)
ENDFOR
RETURN, segbytes
END