PRO fst_focisout_multistr, INFILE=infile, OUTFILE=outfile, FOCISOUT=focisout
;
; Reads focisout file and returns an array of structures, one per record
;
; INFILE  = input filename.  Default = 'focisout.dat'
; OUTFILE = output filename. Default = 'focispect.dat'
; FOCISOUT = named structure to receive output structure (optional).
;
; 17-May-06     Initial version (ghurford@ssl.berkeley.edu)
; 19-May-06 gh  Renamed first properly working version.
;               Change default input to 'focisoutemp.dat'
;               Change default output to 'focisout.dat'
;               Add FOCISOUT keyword.
; 25-May-06 gh  Correct bug in call to fst_focisout_read.
; 24-Jul-06 dg/gh CLOSE and FREE_LUN the input file.
; 28-jUL-06 gh  Add provision for multiband files with uniform sequencing.
; 28 Jul 06 sw  playing with fst_focisout_reformat.pro
;
DEFAULT, infile,    'focisoutemp.dat'
DEFAULT, outfile,   'focispect.dat'
;
; Process the first record to set things up.
input = fst_focisout_read (lunin, /NEWFILE, FILENAME=infile, ACQHEADER=acqheader, /RECHDR)
; replicate this array to hold the multiple structure, max 999 records
focisout=replicate(input, 999)

infilestat      = FSTAT(lunin)
acqheader       = input.acqheader
timesectemp     = input.timesec
pwrspectemp     = input.pwrspect                            ; ntime x nfreq x nch array
corspectemp     = input.corspect
nfreq1          = N_ELEMENTS(input.freqmhz)                 ; number of frequencies in one record
ntime1          = N_ELEMENTS(input.timesec)                 ; number of times in one record
nch             = N_ELEMENTS(input.pwrspect)/nfreq1/ntime1
nap             = N_ELEMENTS(input.corspect)/nfreq1/ntime1
nr              = 0                                         ; number of records read so far
eofflag         = EOF(lunin)
;
; Begin loop over subsequent records
WHILE eofflag EQ 0 DO BEGIN
    input           = fst_focisout_read (lunin, ACQHEADER=acqheader, /RECHDR)
    ; add new structure to old one
    if (size(input,/type) eq 8) then begin
       nr = nr+1
       focisout[nr]=input
    endif
    eofflag     = EOF(lunin)         ; Prepare to exit loop at EOF
ENDWHILE
;
focisout=focisout[0:nr]
; fst_focisout_write, lunout, focisout, FILENAME=outfile, /HEADER, /NEWFILE, /RECHDR
; outfilestat = FSTAT(lunout)
CLOSE, lunin
FREE_LUN, lunin
; CLOSE, lunout
; FREE_LUN, lunout
; PRINT, outfile, outfilestat.size/1.E6, FORMAT="('Reformatted results written to  ', A, F13.6, ' Mbytes')"
RETURN
END
