;+
; NAME:
;     fst_header
; PURPOSE:
;     To read a labview datafile header produced by ACQ software
; CATEGORY:
;     OVRO FST DATA ANALYSIS
; CALLING SEQUENCE:
;     header=fst_header()
; INPUTS:
;
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
;
;
; ROUTINES CALLED:
; OUTPUTS:
; COMMENTS:
; SIDE EFFECTS:
; RESTRICTIONS:
;
; MODIFICATION HISTORY:
;     Written 5-May-2005 by Gelu Nita
;     11-March-2006 made it compatible with the 7-March-2006 major revision of the packet


function fst_header,file,unit=unit,notfree=notfree,comment=comment
    if n_elements(file) eq 0 then begin
     file=dialog_pickfile(filter='ACQ*.dat')
    end
    openr,unit,file,/get_lun,/SWAP_ENDIAN
    t0=systime(/s)
    status = FSTAT(unit)
    file_size=status.size

    ;Read the file header
    header=fst_variant(unit,data_unit)
    free_lun,unit
    unit=data_unit
    heap_gc


    point_lun,-unit,pos
    bytes_left=file_size-pos

    ;Compute number of data records in file

    nrecords=bytes_left/header.recsize

    header=CREATE_STRUCT('FSIZE',file_size,header,'NREC',nrecords)

    ;Print header info if not quiet
    if keyword_set(comment) then begin
     print, 'Header record read in ',systime(/s)-t0, ' seconds'
     print,'Header structure:'
     HELP,header,/STR
    end


    if keyword_set(notfree) eq 0 then free_lun,unit

    return, header

end