pro fst_imgap,filelist,saveps=saveps,noplot=noplot,pm=pm

	nfiles = n_elements(filelist)
    if (nfiles eq 0) then begin
    	filelist = dialog_pickfile(/multi)
        if (filelist[0] eq '') then return
        nfiles = n_elements(filelist)
    endif
    for i = 0, nfiles-1 do begin
        break_file,filelist[0],disk,dir,stem,ext
        if (strupcase(strmid(ext,1,3)) eq 'SAV') then begin
            restore,filelist[i]
        endif else begin
            focisout = fst_focisout_read(lun,filename=filelist[i],/newfile)
            free_lun,lun
        endelse
        f1 = temporary(focisout)
        nt = n_elements(f1.timesec)
        nf = n_elements(f1.freqmhz)
        nchan = f1.acqheader.nchan
        nap = nchan*(nchan-1)/2
        if (i eq 0) then begin
            amp = fltarr(nt*nfiles,nf,nap)
            pha = fltarr(nt*nfiles,nf,nap)
            tp = fltarr(nt*nfiles,nf,nchan)
        endif
      ; cor = fst_frgstop(filelist[i],LO=2.0)
        tp[i*nt:(i+1)*nt-1,*,*] = f1.pwrspect
        amp[i*nt:(i+1)*nt-1,*,*] = abs(f1.corspect)
        pha[i*nt:(i+1)*nt-1,*,*] = lobe(atan(f1.corspect,/phase)*!radeg,/mid)
        if (i eq 0) then ut = f1.timesec else ut = [ut,f1.timesec]
    endfor

    idx = nint((ut - ut[0])/(ut[1]-ut[0]))
    nt2 = max(idx)
    utc = dindgen(nt2)*(ut[1]-ut[0]) + ut[0]
    amp2 = fltarr(nt2,nf,nap)
    pha2 = fltarr(nt2,nf,nap) - 180
    tp2 = fltarr(nt2,nf,nchan)
    amp2[idx,*,*] = amp
    pha2[idx,*,*] = pha
    tp2[idx,*,*] = tp

    DEFAULT, title, ' '
    DEFAULT, display_size, [0.625,0.625]
    DEVICE, get_screen_size=xyscreen
    xysize  = display_size*xyscreen

    channels= INDGEN(nchan)+1                       ; channel numbers to be processed (=1,2,3)
    nch     = N_ELEMENTS(channels)                  ; number of channels to be processed
    ich     = channels -1                           ; vector of indices for channels to be processed
    nap     = nch*(nch-1)/2                         ; number of antenna pairs
    iap     = [[0,1], [1,2], [2,0]]                 ; indices for antenna pairs.

    ; prepare the plotman obj, spectrogram obj
    pm = plotman(/multi,colortable=39)
	; make the total power spectrogram
    for NC = 0, nch-1 DO BEGIN
   		ic = channels[nc]-1
   		lim = median(tp2[*,*,nc])*10.
		title = 'Total Power   '+f1.acqheader.chandesc[ic]
		if nc eq 0 then o = make_spectrogram(tp2[*,*,nc]<lim,utc,f1.freqmhz) $
            else		o -> set, spectrogram_in=tp2[*,*,nc]<lim, time_axis=utc, spectrum_axis=f1.freqmhz
		o -> set, wxsize=xysize[0], wysize=xysize[1], xstyle=3, ystyle=3, title=title, $
				  xtitle='TIME (UT)', ytitle='FREQ (MHz)', interpolate=0, $
				  display_size=display_size, plotman_obj=pm, paneldesc=title
		o -> plotman, plotman_obj=pm
	ENDFOR
	; make the correlated amplitude spectrogram
    FOR nb = 0, nap-1 DO BEGIN
	    jc = iap[*,nb]
   		lim = median(amp2[*,*,nb])*10.
	    title = 'Correlated Amplitude   '+f1.acqheader.chandesc[jc[0]]+'x'+f1.acqheader.chandesc[jc[1]]
	    o -> set, spectrogram_in=amp2[*,*,nb]<lim, time_axis=utc, spectrum_axis=f1.freqmhz
	    o -> set, wxsize=xysize[0], wysize=xysize[1], xstyle=3, ystyle=3, title=title, $
	              xtitle='TIME (UT)', ytitle='FREQ (MHz)', interpolate=0, $
	              display_size=display_size, plotman_obj=pm, paneldesc=title
	    o -> plotman, plotman_obj=pm
	ENDFOR
	; make the correlated phase spectrogram
	FOR nb = 0, nap-1 DO BEGIN
	    jc = iap[*,nb]
		title = 'Correlated Phase   '+f1.acqheader.chandesc[jc[0]]+'x'+f1.acqheader.chandesc[jc[1]]
		o -> set, spectrogram_in=pha2[*,*,nb], time_axis=utc, spectrum_axis=f1.freqmhz
		o -> set, wxsize=xysize[0], wysize=xysize[1], xstyle=3, ystyle=3, title=title, $
		          xtitle='TIME (UT)', ytitle='FREQ (MHz)', interpolate=0, $
				  display_size=display_size, plotman_obj=pm, paneldesc=title
		o -> plotman, plotman_obj=pm
	ENDFOR

    if keyword_set(saveps) then begin
		fst_save_panels,pm,filename=stem+'.ps'
    endif
    obj_destroy,o
    if keyword_set(noplot) then obj_destroy,pm

return
end
