;+
; NAME:
;       fst_psys
;
; PURPOSE:
;       To read and display the FST PSYS data for a given scan
; CATEGORY:
;       FST data analysis
;
; CALLING SEQUENCE:
;        fst_psys,[file,psys]
;
; INPUTS:
;      file: name of a file containing one scan PSYS data in LabVIEW variant format
; OUTPUTS:
;      psys: a structure containg the PSYS data for each observed channel
; KEYWORD PARAMETERS:
;
; ROUTINES CALLED:
;      fst_lv2idl, fst_ut, plotman, fst_getpsys
; REFERENCE:
;
; RESTRICTIONS:
;
; DISCLAIMER:
;
; MODIFICATION HISTORY:
;       Written by:  Gelu M. Nita (gnita@njit.edu),April 7, 2006
;       18-April-2006 GN: Changed code to read the newly introduced PSYS
;                         file format while keeping backward compatibility.
;
;-
pro fst_psys,file,psys=psys,variant=variant
if n_elements(psys) ne 0 then tempvar = SIZE(TEMPORARY(psys))
if n_elements(file) eq 0 then file=dialog_pickfile(filter='PSYS*.lv')
openr,unit,file,/swap_if_little,/get_lun
status=fstat(unit)
if status.mtime le 1145464620 then variant=1
if keyword_set(variant) then begin
 psys=fst_variant(unit)
endif else begin
  template={chanid:0l,timestamp:0d,data:0d}
  fst_getpsys, unit,psys=psys
end
free_lun,unit
for i=1,4 do begin
  id=where(psys.chanid eq i,count)
  if count gt 0 then begin
   newacq=id(where(id-shift(id,1) ne 1))
   data=psys[id].data
   timestamp=psys[id].timestamp
   if n_elements(ps) eq 0 then begin
    ps=Create_Struct(string(i,format="('PSYS',i1)"),Create_Struct('chanid',i,'newacq',newacq,'timestamp',timestamp,'data',data))
   endif else begin
    ps=Create_Struct(ps,Create_Struct(string(i,format="('PSYS',i1)"),Create_Struct('chanid',i,'newacq',newacq,'timestamp',timestamp,'data',data)))
   endelse
  end
end
psys=temporary(ps)
plotman_obj = plotman(/multi)
starttime=fst_ut(psys.(0).timestamp[0],utbase=utbase)
for i=0,n_tags(psys)-1 do begin
	status=plotman_obj -> setdefaults (input=[[fst_ut(psys.(i).timestamp)],[psys.(i).data]],$
	       plot_type='utplot',utbase=utbase,ytitle=string(psys.(i).chanid,format="('PSYS ',i1)"),$
	       WXSIZE=500,WySIZE=300)
	plotman_obj -> new_panel, string(psys.(i).chanid,format="('PSYS',i1)")
end
end