;+
; NAME:
;     ACQ_SegmentDescriptor2sec
; PURPOSE:
;     To convert the segment descriptor structure provided by an Acqiris digitizer
;     in seconds relative to the start of the acquisition (first valid trigger)
; CATEGORY:
;     OVRO FST DATA ANALYSIS
; CALLING SEQUENCE:
;     FST_SegmentDescriptor2sec(SegmentDescriptor)
; INPUTS:
;     SegmentDescriptor={HORPOS:double,TimeStampHi:long, TimeStampLo:long}
;
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
;
;
; ROUTINES CALLED:
; OUTPUTS:double precision or seconds since the start of the acquisition
;           represented as a double float or as an 128bit integer in units of 1/(2^64) seconds
; COMMENTS:
;
; SIDE EFFECTS:
; RESTRICTIONS:
;
; MODIFICATION HISTORY:
;     Written 27-Jan-2005 by Gelu Nita
;
;-
;--------------------------------------------------------------------------

function FST_SegmentDescriptor2sec, SegmentDescriptor, u128=u128
 ul64=(ishft(ulong64(SegmentDescriptor.TimeStampHi),32)+ SegmentDescriptor.TimeStampLo) ;(units of picoseconds)
 result=SegmentDescriptor.HorPos+double(ul64)*1e-12; seconds
 if keyword_set(u128) then begin
  sec=ulong64(floor(result,/L64))
  frac=result-double(sec)
  frac=ulong64(frac*2d^64)
  result=[sec,frac]
 end
 return,result
end
