PRO fst_spectrogram_display, dynaspec, timems, freqmhz, t0, TITLE=title, DISPLAY_SIZE=display_size, LINEAR=linear, $
							plotman_obj=plotman_obj,paneldesc=paneldesc
;
; Handles the interfacing to use the spectrogram object to display an FST spectrogram.
;
; dynaspec is the 2 dimensional (ntime x nfreq) array to be displayed.  Elements are > 0.
; timems and freqmhz are the time and frequency vectors with ntime and nfreq elements respectively
; title is a string that will appear above spectrogram
; display_size is a 2-element vector defining the [x,y] fraction of current screen to be used for display.
; /LINEAR displays dynaspec as is.  Default is to display LOG(dynaspec)
;
; 16-May-05		Initial version - uses old interface protocol (ghurford@ssl.berkeley.edu)
; 21-Jun-05 gh	Set colortable and smooth_spec at as part of one overall set command.
; 22-Jun-05 gh	Add TITLE and DISPLAY_SIZE keywords.
;					Substitute interpolate for smooth_spec parameter.
; 23-Jun-05 gh	Minor debugging
; 16-Aug-05 gh	Add LINEAR keyword
; 25-Feb-06 gh  Change time units to milliseconds.
; 22-May-06 gh  Add t0 argument and change timeaxis to absolute units
;
DEFAULT, title, ' '
DEFAULT, display_size, [0.625,0.625]
DEVICE, get_screen_size=xyscreen
xysize 	= display_size*xyscreen
IF KEYWORD_SET(linear) EQ 0 THEN o 		= make_spectrogram( ALOG10(dynaspec), timems, freqmhz )		$
							ELSE o		= make_spectrogram(         dynaspec, timems, freqmhz )
o->set, wxsize=xysize[0], WYSIZE=xysize[1]
o -> set,  XSTYLE=3, YSTYLE=3, XTITLE=STRMID(ANYTIM(t0,/yohKOH),0,9), $
		 YTITLE='FREQ (MHz)', TITLE=title,  COLORTABLE=39, INTERPOLATE=0
o -> plotman,  plotman_obj=plotman_obj,paneldesc=paneldesc
obj_destroy, o
RETURN
END