;+
; NAME:
;       fst_ut
;
; PURPOSE:
;       To translate a LabVIEW timestamp
; CATEGORY:
;       FST data analysis
;
; CALLING SEQUENCE:
;       todaysec=fst_ut(lv_timestamp,utbase=utbase)
;
; INPUTS:
;       lv_timestamp: a LabVIEW timestamp (double)
;                     as seconds elapsed relative to
;                     1904/1/1 00:00:00.000 UT
; OUTPUTS:
;       seconds elapsed from the start of current day
; KEYWORD OR OPTIONAL PARAMETERS:
;       utbase: set this keyword to retrieve a UTBASE string to be used with UTPLOT
; ROUTINES CALLED:
;       fst_lv2idl, fst_todaysec, plotman
; REFERENCE:
;
; RESTRICTIONS:
;
; DISCLAIMER:
;
; MODIFICATION HISTORY:
;       Written by:  Gelu M. Nita (gnita@njit.edu),April 8, 2006
;-
function fst_ut,lv_timestamp,utbase=utbase
;Compute Julian day
 day = (julday(1,1,1904)-0.5d)+(lv_timestamp/86400d)
;Compute today seconds.
 todaysec=((day-0.5d) mod 1d)*86400d
;Compute today utbase
 CALDAT, day, Month , Day , Year , Hour , Minute , Second
 utbase=string(year,month,day,format="(i4,'/',i2.2,'/',i2.2)")
 return,todaysec
end