FUNCTION fst_focisout_read, lunh, ACQHEADER=acqheader, FILENAME=filename, NEWFILE=newfile, RECHDR=rechdr
;
; Returns a structure containing FOCIS analysis results.
; Returns -1 when EOF is encountered at start of a read.
;
; lunh is a named variable that preserves the unit number in calling program.
;
; KEYWORDS: acqheader  Receives the acqheader structure.  If specified and there is no acqheader, then this value is substituted.
;           newfile    If set, opens a new file.
;           filename   Name of file containing focisout data.  Defaults to 'focisout.dat'
;
; 15-May-06 Initial version. (ghurford@ssl.berkeley.edu)
;
; if acqheader is specified, opens the file and reads returns acqheader structure as well.
;
DEFAULT, filename, 'focisout.dat'
DEFAULT, newfile, 0
focisdata_dim    = INTARR(6)
;
IF newfile NE 0 THEN OPENR, lunh, /GET_LUN, filename
eofflag = EOF(lunh)
IF eofflag THEN RETURN, -1                  ; Return with a -1 at end of file.
READU, lunh, focisdata_dim
IF focisdata_dim[0] GT 0 THEN BEGIN
    savdat = BYTARR(focisdata_dim[0])
    READU,          lunh, savdat
    OPENW,          luntemp, 'tempo.sav', /GET_LUN   ; Write the binary image of the .sav file to a temporary separate file.
    WRITEU,         luntemp, savdat
    CLOSE,          luntemp
    FREE_LUN,       luntemp
    RESTORE,        'tempo.sav'                    ; Then restore 'acqheader' from the temporary file
    FILE_DELETE,    'tempo.sav'
ENDIF
;
ntimes      = focisdata_dim[1]
nfreq       = focisdata_dim[2]
nch         = focisdata_dim[3]
nap         = focisdata_dim[4]
nhr         = focisdata_dim[5]
focisdata    = { timesec: DBLARR(ntimes),               $
                 freqmhz: FLTARR(nfreq),                 $
                pwrspect: FLTARR(ntimes, nfreq, nch),    $
                corspect: COMPLEXARR(ntimes, nfreq, nap)   }
READU, lunh, focisdata
if keyword_set(rechdr) then begin
   ; read in the record header
   recdat = BYTARR(nhr)
   READU,          lunh, recdat
   OPENW,          luntemp, 'tempo.sav', /GET_LUN   ; Write the binary image of the .sav file to a temporary separate file.
   WRITEU,         luntemp, recdat
   CLOSE,          luntemp
   FREE_LUN,       luntemp
   RESTORE,        'tempo.sav'                    ; Then restore 'recheader' from the temporary file
   FILE_DELETE,    'tempo.sav'
   focisout    = { acqheader: acqheader,           $
                     timesec: focisdata.timesec,   $
                     freqmhz: focisdata.freqmhz,   $
                    pwrspect: focisdata.pwrspect,  $
                    corspect: focisdata.corspect,  $
                   recheader: recheader }
endif else focisout    = { acqheader: acqheader,           $
                  timesec: focisdata.timesec,   $
                  freqmhz: focisdata.freqmhz,   $
                 pwrspect: focisdata.pwrspect,  $
                 corspect: focisdata.corspect   }
RETURN, focisout
END
