PRO fst_focisout_write, lunh, focisout, FILENAME=filename, NEWFILE=newfile, HEADER=header, RECHDR=rechdr
;
; Procedure to write focisout data to a binary file.
; If /NEWFILE is set, opens the output file.
; IF /HEADER is set, writes the acqheader information as well.
; IF /RECHDR is set, writes the recheader information as well.
; FILENAME = name of output file.  Default is 'focisout.dat'
;
; 15-May-06   Initial version. (ghurford@ssl.berkeley.edu)
;
; For header, first generate a binary image of the .SAV file for acqheader
;
DEFAULT, filename, 'focisout.dat'
;
; If ACQHEADER is present, write a record corresponding to an image of an acqheader .SAV
nhb = 0
IF KEYWORD_SET(newfile) THEN OPENW,  lunh, /GET_LUN, filename        ; Open the binary output file
IF KEYWORD_SET(header) THEN BEGIN
    acqheader = focisout.acqheader
    SAVE, acqheader, FILE='temp.sav'        ; Save acqheader temporarily as a separate file.
    OPENU, lun1, /GET_LUN, 'temp.sav'       ; Open the save file containing acqheader as if it were unformatted data.
    stat = FSTAT(lun1)
    nhb  = stat.size
    savdat = BYTARR(nhb)                     ; savdat will be a binary image of the .sav file
    READU, lun1, savdat
    CLOSE, lun1
    FILE_DELETE, 'temp.sav'
    FREE_LUN, lun1
ENDIF
;
; Get dimensional information on the data structure and write it at start of output file.
ntimes  = N_ELEMENTS(focisout.timesec)
nfreq   = N_ELEMENTS(focisout.freqmhz)
nch     = N_ELEMENTS(focisout.pwrspect)/nfreq/ntimes
nap     = N_ELEMENTS(focisout.corspect)/nfreq/ntimes
focisdata_dim    = INTARR(6)
focisdata_dim[0] = nhb
focisdata_dim[1] = ntimes
focisdata_dim[2] = nfreq
focisdata_dim[3] = nch
focisdata_dim[4] = nap
focisdata_dim[5] = 0
if keyword_set(rechdr) then begin
   ; get size of record header as we did for ACQ header
   recheader = focisout.recheader
   SAVE, recheader, FILE='temp.sav'        ; Save acqheader temporarily as a separate file.
   OPENU, lun1, /GET_LUN, 'temp.sav'       ; Open the save file containing acqheader as if it were unformatted data.
   rstat = FSTAT(lun1)
   nhr  = rstat.size
   recdat = BYTARR(nhr)                     ; recdat will be a binary image of the .sav file
   READU, lun1, recdat
   CLOSE, lun1
   FILE_DELETE, 'temp.sav'
   FREE_LUN, lun1
   focisdata_dim[5] = nhr
endif

WRITEU, lunh, focisdata_dim
IF KEYWORD_SET(header) THEN WRITEU, lunh, savdat
focisdata = {timesec: focisout.timesec,     $
             freqmhz: focisout.freqmhz,     $
            pwrspect: focisout.pwrspect,    $
            corspect: focisout.corspect     }
WRITEU, lunh, focisdata
if keyword_set(rechdr) then WRITEU, lunh, recdat
RETURN
END
